# django-salesforce
#
# by Phil Christensen
# (c) 2012-2013 Freelancers Union (http://www.freelancersunion.org)
# See LICENSE.md for details
#

"""
Multi-database support for the Django admin.
Not important yet - replaced by an empty stub.
"""
from django.contrib.admin.options import ModelAdmin as RoutedModelAdmin  # NOQA pylint:disable=unused-import
